import pandas as pd
from statsmodels.tsa.arima.model import ARIMA
import matplotlib.pyplot as plt
from statsmodels.graphics.tsaplots import plot_acf, plot_pacf
import warnings

warnings.filterwarnings("ignore")
pd.set_option("display.max_columns", None)
pd.set_option("display.max_rows", None)
pd.set_option("display.width", None)

# 设置Matplotlib默认字体为SimHei
plt.rcParams['font.sans-serif'] = ['SimHei']
plt.rcParams['axes.unicode_minus'] = False  # 用于显示负号
# 读取时间序列数据
data = pd.read_excel(r'D:\text.xlsx')# 假设数据保存在 steam_flow_data.csv 中
data['时间'] = pd.to_datetime(data['时间'])  # 将日期列转换为日期时间格式
data.set_index('时间', inplace=True)

# 检查数据的自相关性和偏自相关性
plot_acf(data['蒸汽压力'])
plot_pacf(data['蒸汽压力'])
plt.show()

# 拟合 ARIMA 模型
# 选择合适的 ARIMA(p,d,q) 参数，这里使用了简单的参数，实际中可能需要调优
model = ARIMA(data['蒸汽压力'], order=(2,1,2))
model_fit = model.fit()

# 打印模型的概要信息
print(model_fit.summary())

# 绘制预测结果
plt.figure(figsize=(10,6))
plt.plot(data.index, data['蒸汽压力'], label='实际')
plt.plot(data.index + pd.DateOffset(months=1), model_fit.fittedvalues, color='red', label='预测值')
plt.legend()
plt.title('ARIMA模型预测')
plt.xlabel('时间')
plt.ylabel('蒸汽压力')
plt.show()

print("未来一个月内的蒸汽生产总量：%.2f"%sum(model_fit.fittedvalues))
